/* LIBGIMP - The GIMP Library
 * Copyright (C) 1995-2003 Peter Mattis and Spencer Kimball
 *
 * gimptexttool_pdb.c
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl */

#include "config.h"

#include "gimp.h"


/**
 * SECTION: gimptexttool
 * @title: gimptexttool
 * @short_description: Functions for controlling the text tool.
 *
 * Functions for controlling the text tool.
 **/


/**
 * gimp_text_fontname:
 * @image_ID: The image.
 * @drawable_ID: The affected drawable: (-1 for a new text layer).
 * @x: The x coordinate for the left of the text bounding box.
 * @y: The y coordinate for the top of the text bounding box.
 * @text: The text to generate (in UTF-8 encoding).
 * @border: The size of the border.
 * @antialias: Antialiasing.
 * @size: The size of text in either pixels or points.
 * @size_type: The units of specified size.
 * @fontname: The name of the font.
 *
 * Add text at the specified location as a floating selection or a new
 * layer.
 *
 * This tool requires a fontname matching an installed PangoFT2 font.
 * You can specify the fontsize in units of pixels or points, and the
 * appropriate metric is specified using the size_type argument. The x
 * and y parameters together control the placement of the new text by
 * specifying the upper left corner of the text bounding box. If the
 * specified drawable parameter is valid, the text will be created as a
 * floating selection attached to the drawable. If the drawable
 * parameter is not valid (-1), the text will appear as a new layer.
 * Finally, a border can be specified around the final rendered text.
 * The border is measured in pixels. Parameter size-type is not used
 * and is currently ignored. If you need to display a font in points,
 * divide the size in points by 72.0 and multiply it by the image's
 * vertical resolution.
 *
 * Returns: The new text layer or -1 if no layer was created.
 **/
gint32
gimp_text_fontname (gint32        image_ID,
                    gint32        drawable_ID,
                    gdouble       x,
                    gdouble       y,
                    const gchar  *text,
                    gint          border,
                    gboolean      antialias,
                    gdouble       size,
                    GimpSizeType  size_type,
                    const gchar  *fontname)
{
  GimpValueArray *args;
  GimpValueArray *return_vals;
  gint32 text_layer_ID = -1;

  args = gimp_value_array_new_from_types (GIMP_TYPE_IMAGE_ID,
                                          GIMP_TYPE_DRAWABLE_ID,
                                          G_TYPE_DOUBLE,
                                          G_TYPE_DOUBLE,
                                          G_TYPE_STRING,
                                          GIMP_TYPE_INT32,
                                          G_TYPE_BOOLEAN,
                                          G_TYPE_DOUBLE,
                                          GIMP_TYPE_SIZE_TYPE,
                                          G_TYPE_STRING,
                                          G_TYPE_NONE);
  gimp_value_set_image_id (gimp_value_array_index (args, 0), image_ID);
  gimp_value_set_drawable_id (gimp_value_array_index (args, 1), drawable_ID);
  g_value_set_double (gimp_value_array_index (args, 2), x);
  g_value_set_double (gimp_value_array_index (args, 3), y);
  g_value_set_string (gimp_value_array_index (args, 4), text);
  g_value_set_int (gimp_value_array_index (args, 5), border);
  g_value_set_boolean (gimp_value_array_index (args, 6), antialias);
  g_value_set_double (gimp_value_array_index (args, 7), size);
  g_value_set_enum (gimp_value_array_index (args, 8), size_type);
  g_value_set_string (gimp_value_array_index (args, 9), fontname);

  return_vals = gimp_run_procedure_with_array ("gimp-text-fontname",
                                               args);
  gimp_value_array_unref (args);

  if (g_value_get_enum (gimp_value_array_index (return_vals, 0)) == GIMP_PDB_SUCCESS)
    text_layer_ID = gimp_value_get_layer_id (gimp_value_array_index (return_vals, 1));

  gimp_value_array_unref (return_vals);

  return text_layer_ID;
}

/**
 * gimp_text_get_extents_fontname:
 * @text: The text to generate (in UTF-8 encoding).
 * @size: The size of text in either pixels or points.
 * @size_type: The units of specified size.
 * @fontname: The name of the font.
 * @width: (out): The width of the specified font.
 * @height: (out): The height of the specified font.
 * @ascent: (out): The ascent of the specified font.
 * @descent: (out): The descent of the specified font.
 *
 * Get extents of the bounding box for the specified text.
 *
 * This tool returns the width and height of a bounding box for the
 * specified text string with the specified font information. Ascent
 * and descent for the specified font are returned as well. Parameter
 * size-type is not used and is currently ignored. If you need to
 * display a font in points, divide the size in points by 72.0 and
 * multiply it by the vertical resolution of the image you are taking
 * into account.
 *
 * Returns: TRUE on success.
 **/
gboolean
gimp_text_get_extents_fontname (const gchar  *text,
                                gdouble       size,
                                GimpSizeType  size_type,
                                const gchar  *fontname,
                                gint         *width,
                                gint         *height,
                                gint         *ascent,
                                gint         *descent)
{
  GimpValueArray *args;
  GimpValueArray *return_vals;
  gboolean success = TRUE;

  args = gimp_value_array_new_from_types (G_TYPE_STRING,
                                          G_TYPE_DOUBLE,
                                          GIMP_TYPE_SIZE_TYPE,
                                          G_TYPE_STRING,
                                          G_TYPE_NONE);
  g_value_set_string (gimp_value_array_index (args, 0), text);
  g_value_set_double (gimp_value_array_index (args, 1), size);
  g_value_set_enum (gimp_value_array_index (args, 2), size_type);
  g_value_set_string (gimp_value_array_index (args, 3), fontname);

  return_vals = gimp_run_procedure_with_array ("gimp-text-get-extents-fontname",
                                               args);
  gimp_value_array_unref (args);

  *width = 0;
  *height = 0;
  *ascent = 0;
  *descent = 0;

  success = g_value_get_enum (gimp_value_array_index (return_vals, 0)) == GIMP_PDB_SUCCESS;

  if (success)
    {
      *width = g_value_get_int (gimp_value_array_index (return_vals, 1));
      *height = g_value_get_int (gimp_value_array_index (return_vals, 2));
      *ascent = g_value_get_int (gimp_value_array_index (return_vals, 3));
      *descent = g_value_get_int (gimp_value_array_index (return_vals, 4));
    }

  gimp_value_array_unref (return_vals);

  return success;
}
